/***************************************************************************
                 Example of the new features of DSLib v1.1.0
                             -------------------
    begin                : Tue Jun 13 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dynamicsystem.h"
#include "loop.h"
#include "systemvariable.h"
#include "settingsharedmemory.h"
#include "functionoutput.h"
#include "summingpoint.h"
#include "gain.h"
#include "integrator.h"
#include <fstream>

#define LOGGING_SAMPLING_RATE 0.1
#define INPUT_OFFSET 250 * 0x100000

int main(int argc, char *argv[])
{
	// Defining a dynamic system
	CDynamicSystem DynamicSystem;
	bool bNoError = true;
	bNoError = DynamicSystem.SetTime(10000000) && bNoError;
	bNoError = DynamicSystem.SetTime(20000 * DynamicSystem.Time()) && bNoError;
	bNoError = DynamicSystem.SetIncludes("#include <math.h>") && bNoError;
	
	CLoop* pLLoop = new CLoop();
	bNoError = pLLoop->SetName("L") && bNoError;
	bNoError = pLLoop->SetScheduling(CLoop::PERIOD) && bNoError;
	bNoError = pLLoop->SetPeriod(1000000000 * 0.01) && bNoError;
	bNoError = pLLoop->SetPriority(0) && bNoError;
	bNoError = DynamicSystem.AddComponent(pLLoop) && bNoError;
	
	// System Input
	CSystemVariable* pSVSystemInput = new CSystemVariable();
	bNoError = bNoError && pSVSystemInput->SetName("SystemInput");
	bNoError = bNoError && pSVSystemInput->SetType("float");
	bNoError = bNoError && pSVSystemInput->SetInitialization(
		"SystemInput = 0;");
	bNoError = bNoError && DynamicSystem.AddComponent(pSVSystemInput);
	// Setting shared memory
	CSettingSharedMemory* pSSMSystemInput;
	pSSMSystemInput = new CSettingSharedMemory();
	bNoError = bNoError & pSSMSystemInput->SetOffset(INPUT_OFFSET);
	bNoError = pSVSystemInput->AddComponent(pSSMSystemInput) && bNoError;
	
	// System Input to Reference
	CFunction* pFSysIn2Reference = new CFunction();
	bNoError = pFSysIn2Reference->SetName("SysVar2Reference") && bNoError;
	bNoError = pFSysIn2Reference->SetCode("Reference = SystemInput;");
	bNoError = pLLoop->AddComponent(pFSysIn2Reference);
	// Outputs
	CFunctionOutput* pOReference = new CFunctionOutput();
	bNoError = pOReference->SetName("Reference") && bNoError;
	bNoError = pOReference->SetType("float") && bNoError;
	bNoError = pFSysIn2Reference->AddComponent(pOReference) && bNoError;

	// Logging Fifo
	CLoggingFifo* pLFReference = new CLoggingFifo();
	bNoError = pLFReference->SetNumber(0) && bNoError;
	bNoError = pLFReference->SetSamplingRate(LOGGING_SAMPLING_RATE) &&
		bNoError;
	bNoError = pLFReference->SetSize(sizeof(float) * 20) && bNoError;
	bNoError = pOReference->AddComponent(pLFReference) && bNoError;

	// Summing Point
	CSummingPoint* pSummingPoint = new CSummingPoint();
	bNoError = pSummingPoint->SetName("SummingPoint") && bNoError;
	bNoError = pSummingPoint->SetNumberOfInputs(2) && bNoError;
	bNoError = pSummingPoint->SetIOsType("float") && bNoError;
	bNoError = pSummingPoint->Input(1)->Connect(pOReference) && bNoError;
	bNoError = pLLoop->AddComponent(pSummingPoint) && bNoError;
	
	// Gain
	CGain* pGain = new CGain();
	bNoError = pGain->SetName("Gain") && bNoError;
	bNoError = pGain->SetConstantFactor(0.1) && bNoError;
	bNoError = pGain->Input()->Connect(pSummingPoint->Output()) && bNoError;
	bNoError = pGain->SetIOsType("float") && bNoError;
	bNoError = pLLoop->AddComponent(pGain) && bNoError;
	
	// Integrator
	CIntegrator* pIntegrator = new CIntegrator();
	bNoError = pIntegrator->SetName("Integrator") && bNoError;
	bNoError = pIntegrator->Input()->Connect(pGain->Output()) && bNoError;
	bNoError = pIntegrator->SetIOsType("float") && bNoError;
	bNoError = pLLoop->AddComponent(pIntegrator) && bNoError;
	
	// LoggingFifo
	CLoggingFifo* pLFSystemOutput = new CLoggingFifo();
	bNoError = pLFSystemOutput->SetNumber(1) && bNoError;
	bNoError = pLFSystemOutput->SetSamplingRate(LOGGING_SAMPLING_RATE) &&
		bNoError;
	bNoError = pLFSystemOutput->SetSize(sizeof(float) * 20) && bNoError;
	bNoError = pIntegrator->Output()->AddComponent(pLFSystemOutput) && bNoError;
	
	bNoError = pSummingPoint->Input(2)->Connect(pIntegrator->Output()) && bNoError;
	bNoError = pSummingPoint->Input(2)->SetNegated(true) && bNoError;
	
	// Source code generation
	bNoError = DynamicSystem.ArrangeAll() && bNoError;

	if (!bNoError)
	{
		CErrors Errors;
		DynamicSystem.ReportErrors(Errors);
		CErrors::ConstIterator ErrorsIterator;
		for (ErrorsIterator = Errors.Begin(); ErrorsIterator != Errors.End();
			ErrorsIterator++)
		{
			cout << (*ErrorsIterator).Number() << endl;
			cout << (*ErrorsIterator).Message() << endl;
			cout << (*ErrorsIterator).Location()->Class() << endl;
			cout << (*ErrorsIterator).Location()->Identifier() << endl;
			cout << (*ErrorsIterator).Location()->Container()->Identifier() << endl;
		}
	}

	ofstream FileStream;
	FileStream.open("/home/ivan/project/dslib/rtmodule/testrtmod2.c");
	DynamicSystem.GenerateSource(FileStream);
	FileStream.close();
	DynamicSystem.DeleteObjects();

	return EXIT_SUCCESS;
}
